
#include "cmdLine.h"
#include "intParam.h"
#include "floatParam.h"
#include "stringParam.h"
#include "switchParam.h"
#include "compoundParam.h"


cmdLine::cmdLine(string programName, string description)
{
    m_numParameters = 0;
    m_description = description;
    m_programName = programName;
}

cmdLine::~cmdLine()
{
    for (int index =0; index < m_numParameters; index ++ )
    {
        delete m_parameter[index];
    }
}

    // add an integer parameter
void cmdLine::addParam(const char flag, int *target, const string description, const int options=0, int *numExamples=NULL, int maxExamples=1)
{
    checkIfFlagReused(flag);  // exit(1) if fail test
    m_parameter.push_back(new intParam(flag, target, description, options, 
                numExamples, maxExamples));
    m_numParameters++;
}

    // add an float parameter
void cmdLine::addParam(const char flag, float *target, const string description, const int options=0, int *numExamples=NULL, int maxExamples=1)
{
    checkIfFlagReused(flag);  // exit(1) if fail test
    m_parameter.push_back(new floatParam(flag, target, description, options, 
                numExamples, maxExamples));
    m_numParameters++;
}

    // add a string parameter
void cmdLine::addParam(const char flag, string *target, const string description, 
        const int options=0, int *numExamples=NULL, int MaxExamples=1)
{
    checkIfFlagReused(flag);  // exit(1) if fail test
    m_parameter.push_back(new stringParam(flag, target, description, options, 
                numExamples, MaxExamples));
    m_numParameters++;
}

    // add a switch parameter
void cmdLine::addParam(const char flag, bool *target, const string description, 
        const int options=0, int *numExamples=NULL, int MaxExamples=1)
{
    checkIfFlagReused(flag);  // exit(1) if fail test
    m_parameter.push_back(new switchParam(flag, target, description, options, 
                       numExamples, MaxExamples));
    m_numParameters++;
}

    // add a compound parameter
void cmdLine::addParam(const char flag, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1)
{
    checkIfFlagReused(flag);  // exit(1) if fail test
    m_parameter.push_back(new compoundParam(flag, description, options, 
                numExamples, MaxExamples));
    m_numParameters++;
}

    // ensure that whether we are iterated aggrees with the patameter
void cmdLine::checkAllOrNoneIterated(bool iterated)
{
    if (m_parameter[m_numParameters-1]->isIterated() != iterated)
    {
        m_parameter[m_numParameters-1]->paramError
            ("A compound and its components must be iether all \n"
                          "iterated or all not iterated.");
    }

}

    // add an int component to a compound parameter
void cmdLine::addToCompound(int *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1)
{
    checkAllOrNoneIterated((options &ITERATED) == ITERATED);  // exits if fail

    param *tmp = (new intParam('C', target, description, options,
                numExamples, MaxExamples));
    m_parameter[m_numParameters-1]->addComponent(tmp, MaxExamples);
}

    // add a float component to a compound parameter
void cmdLine::addToCompound(float *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1)
{
    checkAllOrNoneIterated((options &ITERATED) == ITERATED);  // exits if fail

    param *tmp = (new floatParam('C', target, description, options,
                numExamples, MaxExamples));
    m_parameter[m_numParameters-1]->addComponent(tmp, MaxExamples);
}

    // add a string component to a compound parameter
void cmdLine::addToCompound(string *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1)
{
    checkAllOrNoneIterated((options &ITERATED) == ITERATED);  // exits if fail

    param *tmp = (new stringParam('C', target, description, options,
                numExamples, MaxExamples));
    m_parameter[m_numParameters-1]->addComponent(tmp, MaxExamples);
}

    // add a switch component to a compound parameter
void cmdLine::addToCompound(bool *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1)
{
    m_parameter[m_numParameters-1]->paramError
        ("You can't have a switch parameter as a "
         "component of a compound parameter.");
}

    // print a usage line
void
cmdLine::usage() const
{
    cerr << m_programName << " ";
    for (int i=0; i< m_numParameters; i++)
    {  // print usage for each parameter
        m_parameter[i]->usage();
    }
    cerr << endl;
}

    // has this flag been used before?   exit(1) if fail test
void 
cmdLine::checkIfFlagReused(char newFlag)
{
    for (int i=0; i< m_numParameters; i++)
    {
        if (m_parameter[i]->flagReused(newFlag))
        {
            cerr << "Error: Command line flag \"" << newFlag;
            cerr << "\" was used in previous call." << endl;
            exit(1);
        }
    }
}

    // print descriptions of the command line parameters
void
cmdLine::printDescription() const
{
    cerr << m_description << endl;
    for (int i=0; i< m_numParameters; i++)
    {  // print each description
        m_parameter[i]->printDescription();
    }
    cerr << endl;
}

    // print defaults for command line arguments
void
cmdLine::printDefault()  const
{
    cerr << "default:" << endl;
    cerr << m_programName << " ";
    for (int i=0; i< m_numParameters; i++)
    {  // print each default argument.
        m_parameter[i]->printDefault();
        cerr << " ";
    }
    cerr << endl;

}

    // fetch the parameter values out of the argv list.  This method must
    // be called after the parameters have been constructed.
void 
cmdLine::getArgs(const int argc, char *argv[])
{
    int arg=1;

    for (int i=0; i< m_numParameters; i++)  
	{ // ensure compound parameteres have at least one component
        m_parameter[i]->checkWrongNumComponents();
	}

    while (arg < argc)
    { // process each command line argument
        if (isSynopsis(argv[arg]))
        {  // print help info and bail out
            usage();
            if (isHelp(argv[arg]))
            { // need parameter descriptions as well
                printDescription();
            }
            printDefault();
            exit(1);
        }

        int a=0;
        int parametersUsed=0;
        while (a<m_numParameters && (parametersUsed == 0)) 
        { // see if any of the parameters can use the command line argument
            parametersUsed = m_parameter[a]->useArgs(arg, argc, argv);
            a++;
        }
        arg += parametersUsed;  // mark out where we are up to in argument list

        if (parametersUsed == 0)
        {  // can't use command line argument:  Print help and quit.
            usage();
            printDefault();
            exit(1);
        }
    }

    for (int i=0; i< m_numParameters; i++)
    {  // make sure all required parameters have been supplied.
        if (m_parameter[i]->checkIsUnusedRequired())
        {
            usage();
            exit(1);
        }
    }
}
