



#ifndef CMDLINE_H
#define CMDLINE_H

#include <string>
#include <vector>
//#include "param.h"
/////// begin of included param.h //////////////////
#ifndef PARAM_H
#define PARAM_H

#include <string>

#define REQUIRED 1
#define ITERATED 2
#define NONFLAG  4

// make virtual:
// 1. produce a virtual method that does only what is unique to the sub-classes
//         it should be some kind of set or get method
// 2. call the new virtual method from existing the over-ridden methods
//         the over-ridden methods should become identical as a result
// 3. move over-ridden method into base and delete from sub-classes
//
//
//  abstract base class for paramerter managers
//
class param{
protected: 
    char m_flag;         // the command line flag
    bool m_iterated;     // option  (an iterated parameter)
    bool m_nonFlag;      // option  (a non-flag paramter)
    bool m_required;     // option  (a manditory parameter)
    string m_description;// an online help description
    string m_typeName;   // a string to represent the type.

    bool m_used;         // has this parameter been used?
    int *m_numExamples;
        // value is 0 for a flag, 1 for a normal param, N for an iterated param
        // meaning is "number of instances of argument on command line at 
        // run time"
    int m_localNumExamples;  // a local version in for non-iterated parameters
    int m_maxExamples;  // for interated parameters where the number is limited
	                    // by the size of a target array

    int m_numComponents;
        //value is 0 for switches, 1 for non-compound, >=1 for compound
        //meaning is "number of components in parameter"

    void init(int options); // set up the option switches

public:
    param();
    param(const char flag, const string description, const int options);
    param(const char flag, const string description, const int options, int *numExamples, const int MaxExamples);
    virtual ~param();
	//~param() { m_numInstances--; cerr <<m_numInstances << endl; } ;

    // use the argument if we can and return number of arguments actually used
    int useArgs(const int param, const int argc, char *argv[]);
    bool checkIsUnusedRequired() const;
    virtual bool checkWrongNumComponents() const {return false;};
    bool isIterated() const {return m_iterated;};
    bool flagReused(const char newFlag) const {return (newFlag == m_flag);};
	// how to use this parameter
    void usage() const;
	// print description of this parameter
    void printDescription() const;
    virtual void describeParam() const {cerr << m_description;};
    
	// print the cmd line arguments that would set this parameter to its
	// default
    void printDefault() const;
	void paramError(string message) const;

	virtual void addComponent(param *component,  const int MaxExamples=1);
    virtual void setParam(char **arg)=0;

    virtual void printValue() const  =0 ;
    virtual void printType() const {cerr << m_typeName;} ;
private:
    static int m_numClassInstances;  // automatically set to zero
	static int m_numNonFlagArgs;    // we can only have one non flag argument
    bool canUseFlaggedArg(const int arg, const int argc, char *argv[]) const;
    bool canUseNonFlagArg(const int arg, const int argc, char *argv[]) const;
};
#endif
/////////////// end of included param.h ///////////////////////

// A class for care and feeding of command line parameters
// if things go well it will construct a set of parameteres and
// feed the command line arguments to the parameter set.  
// Otherwise some diagnostic will be printed on the standard error.
// each parameter gets:
//     a description of the parameter for use in diagnostics
//     a flag character (should be a string?)
//     a pointer to the target data item
// an iterated patameter gets a pointer to an integer that will record
//     the number of instances of the command line argument at run time

class cmdLine
{
private: 
    vector<param*> m_parameter;
    int m_numParameters;      // how many there are
    string m_description;     // to print meaning of the program
    string m_programName;     // name used to call program
    int m_numNonFlagArgs;    // we can only have one

public:
    cmdLine(string programName, string description);
    ~cmdLine();

             ////////////////////////////////////////////////////
             // methods to add various types of parameter      //
             ////////////////////////////////////////////////////
    
    // add an integer parameter
    void addParam(const char flag, int  *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int maxExamples=1);
    
    // add a float parameter
    void addParam(const char flag, float  *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int maxExamples=1);

    // add a string parameter
    void addParam(const char flag, string *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a switch parameter
    void addParam(const char flag, bool *target, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a compound parameter
    void addParam(const char flag, const string description, 
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // ensure that whether we are iterated aggrees with the patameter
    void checkAllOrNoneIterated(bool iterated);

    // add an int component to a compound parameter
    void addToCompound(int *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a float component to a compound parameter
    void addToCompound(float *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // add a string component to a compound parameter
    void addToCompound(string *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

    // dont't add a switch component to a compound parameter
    void addToCompound(bool *target, const string description,
            const int options=0, 
            int *numExamples=NULL, const int MaxExamples=1);

             ////////////////////////////////////////////////////////
             // The method that fetches data from the command line //
             ////////////////////////////////////////////////////////
    // fetch the parameter values out of the argv list.  This method must
    // be called after the parameters have been constructed.
    void getArgs(const int argc, char *argv[]);

private:
    // is this command line argument asking for a synopsis report?
    bool isSynopsis(const char *flag) const
        {return ((flag[0] == '-') && (tolower(flag[1]) == 'h'));};

    // is this command line argument asking for synopsis and descriptions?
    bool isHelp(const char *flag){return (strcmp(flag,"-H")==false);};

	// has this flag been used before? This method exits if newFlag is reused.
	void checkIfFlagReused(char newFlag);

    // print descriptions of the command line parameters
    void printDescription() const;

    // print a usage line
    void usage() const;

    // print defaults for command line arguments
    void printDefault() const;
};
#endif
