
#include <string>
#include <iostream>

#include "param.h"

// a couple of class instance counters
int param::m_numClassInstances = 0;
int param::m_numNonFlagArgs = 0;

param::~param()
{
    m_numClassInstances--;
    //cerr << "destruct: " << "number of param objects is ";
    //cerr << m_numClassInstances << endl;
}

param::param(const char flag, const string description, const int options):
    m_flag(flag), m_description(description), m_numExamples(&m_localNumExamples),
    m_maxExamples(1),
    m_numComponents(0)
{
    init(options);

    if (m_iterated)
		paramError("ITERATED option implies extra arguments for count and max");
}

param::param(const char flag, const string description, const int options, int *numExamples, const int MaxExamples):
    m_flag(flag), m_description(description), 
    m_maxExamples(MaxExamples),
    m_numComponents(0)
{
    if (numExamples == NULL)
         m_numExamples = &m_localNumExamples;
    else
         m_numExamples = numExamples;
    init(options);
}

void param::init(int options)
{
    m_numClassInstances++;
    //cerr << "construct: " << "number of param objects is ";
    //cerr << m_numClassInstances << endl;

    if (tolower(m_flag) == 'h')
        paramError("Reserved command line parameter.");
    m_iterated = options & ITERATED;
    m_required = options & REQUIRED;
    if ((options & NONFLAG) != 0)
    {
        m_nonFlag = options & NONFLAG;
        m_numNonFlagArgs++;
        if (m_numNonFlagArgs > 1) 
        { // second nonFlag parameter, but can only have one
			paramError("Trying to create a second non-flag parameter.\n"
					   "             You can only have one.");
        }
    }
    m_used = false;
    *m_numExamples = 0;
}

bool 
param::checkIsUnusedRequired() const
{
    if (!m_used && m_required)
    {
		paramError("Unused required command line argument.");
		return true;
    }
	else
		return false;
}

void 
param::usage()  const
{
    if (!m_required)
        cerr << "[";
    if (m_nonFlag)
    {
        cerr << "[-"; cerr << m_flag; cerr << "] ";
    }
    else
    {
        cerr << "-"; cerr << m_flag; cerr << " ";
    }
	printType();
    if (!m_required)
        cerr << "]";
    if (m_iterated)
        cerr << "...";
    cerr << "  ";
}

void
param::printDescription() const
{
    cerr << "   ";
    cerr << m_flag;
    cerr << " -> ";
	describeParam();
    cerr << endl;
}

void
param::paramError(string message) const
{
	cerr << "Parameter error: -" << m_flag << ": "<< message << endl;
	exit(1);
}

void
param::printDefault() const
{
    //if (!(m_iterated && (*m_numExamples == 0)))
    if (!m_iterated)
    {
        cerr << "-" << m_flag <<  " ";     
        printValue();
    }
}

void 
param::addComponent(param *component,  const int MaxExamples=1)
{
	paramError("Error: you can only call addToCoumpound() just after you \n create a compound parameter. ");
}
// make virtual:
// 1. produce a virtual method that does just what is unique to sub-classes
//         it should be some kind of set or get method
// 2. call new virtual method from existing over-ridden methods
//         overridden methods should become identical
// 3. move over-ridden class into base and delete from sub-classes


// return true if the argument has no flag and we can use it
bool
param::canUseNonFlagArg(const int arg, const int argc, char *argv[]) const
{
    if (arg + m_numComponents -1 < argc)   // there are enough arguments
        if (argv[arg][0] != '-')           // found a non-flag argument 
            if (m_nonFlag)                 // we are a nonFlag parameter
                if (*m_numExamples < m_maxExamples)// can still use a non-flag arg
                //if (!m_used || m_iterated);// can still use a non-flag arg
                    return true;            // we can use the argument
    return false;   // can't use the argument
}


// return true if the argument has a flag and we can use it
bool
param::canUseFlaggedArg(const int arg, const int argc, char *argv[]) const
{
    if (arg + m_numComponents < argc)   // there are enough arguments
        if (argv[arg][0] == '-')        // found an argument flag
            if (m_flag == argv[arg][1]) // it is the right flag
                if (*m_numExamples < m_maxExamples)// can use the flag arg
                    return true;            // we can use the argument
                else
                {
                    cerr << "too many -" << m_flag << " args on command line";
                    cerr << endl;
                    exit(1);
                }
    return false;   // can't use the argument
}

// use the argument if we can, return number of arguments actually used
int
param::useArgs(const int arg, const int argc, char *argv[])
{
    int dataOffset = 0;
    bool canUseArg=false;

    if (canUseNonFlagArg(arg, argc, argv))
    {
        dataOffset = 0;
        canUseArg = true;
    }
    else if (canUseFlaggedArg(arg, argc, argv))
    {
        dataOffset = 1;
        canUseArg = true;
    }

    if (canUseArg)
    {
        setParam(argv + arg + dataOffset);
        m_used = true;
        return dataOffset + m_numComponents;
    }
    else 
        return 0;
}
